@@ -1,30 +1,30 @@
 #include "cuc.h"
-#include <assert.h>
+#include <cassert>
 
 
 #ifdef DARWIN
 #include <sys/param.h>
 #endif
 
-string getCwd()
+std::string getCwdir()
 {
-#ifndef DARWIN
-  char *savepwd2;
-  char *tempbuf;
+
+#if !defined(DARWIN) && !defined(NETBSD) && !defined(OPENBSD)
+  std::string returnMe;
   long tempsize = pathconf (".", _PC_PATH_MAX);
+  char *tempbuf;
   tempbuf = new char[tempsize];
   assert(tempbuf != NULL);
-  if ((savepwd2 = getcwd (tempbuf, (size_t) tempsize)) == NULL)
+  if ((getcwd (tempbuf, (size_t) tempsize)) == NULL)
     {
       cout << "Could not obtain Current Working Directory..." << endl;
       perror ("getcwd()");
       exit (EXIT_FAILURE);
     }
-  delete[] tempbuf;
-  return savepwd2;
-#endif
-
-#ifdef DARWIN
+  returnMe = tempbuf;
+  delete [] tempbuf;
+  return returnMe;
+#else
   char buf[BUFSIZ];
   string savepwd2=getcwd(buf, MAXPATHLEN);
   return savepwd2;
@@ -32,3 +32,4 @@ string getCwd()
 
 
 }
+
