@@ -2,33 +2,30 @@
    This function is called when -turn is given.
    It copies ~/.icpld/log to ~/.icpld/log.old.n, and 
    creates a new log.
+
+   This code has been C:ified to comply with the poor 
+   bastards still running gcc version < 3 (no stringstreams)
+
 */
 
 #include "cuc.h"
 #include <sys/stat.h>
-#include <sstream>
+
 void
 logTurn (std::string logFile)
 {
-  string newLog, tempLog = logFile, num;
+  string newLog;
+  char tempLog[FILENAME_MAX], conv[17];
   struct stat s;
-  stringstream conv;
-  /* It should be a fair assumption, that no one keeps
-     more than 1000 old logs 
-  */
   for (int i = 0; i < 1001 ; i++){ 
-    conv << i;
-    conv >> num;
-    tempLog+=".";
-    tempLog+=num;
-    if (stat(tempLog.c_str(), &s) == -1){
+  strncat (tempLog, logFile.c_str(), strlen(logFile.c_str()));
+    sprintf(conv, ".%d", i);
+    strncat(tempLog, conv, strlen(conv));
+    if (stat(tempLog, &s) == -1){
       newLog = tempLog;
       break;
     }
-    conv.flush();
-    conv.clear();
-    tempLog=logFile;
-
+    strcpy(tempLog, "");
     /* Windows-style warning */
     if (i == 1000){
       cout << "You have reached the maximum amount of logs possible. I'm glad you are not my sys-admin." << endl;
